import 'package:flutter/widgets.dart';
import '../model/configuration.dart';
import '../agent_impl.dart';
import '../model/data_collection_level.dart';
import '../model/platform.dart';
import '../action/interface/root_action.dart';

/// The main API of the flutter plugin which gives access
/// to the functions of the native agents.
abstract class Dynatrace {
  /// Factory which creates a [Dynatrace] object which is able to do the native
  /// communication with the Mobile Agents.
  factory Dynatrace() => DynatraceImpl();

  /// Start function which should be called to start the agent. The [topLevelWidget]
  /// is the widget which is starting the application. The Configuration contains
  /// default values for all properties. reportCrash value is true per default.
  /// monitorWebRequest is true per default.
  Future<void> start(Widget topLevelWidget, {Configuration configuration});

  /// Creating a [DynatraceRootAction] which is able to have child actions. If
  /// you enter [null] or an empty [String] for the [name] you will get a root action
  /// which will be disabled.
  DynatraceRootAction enterAction(String name, {Platform platform});

  /// Report an error with [errorName] and [errorCode] directly without any action
  void reportError(String errorName, int errorCode, {Platform platform});

  /// Report an error which contains a [stacktrace] and will therefor be reported as a real
  /// crash. The error includes [errorName] and [reason] as well.
  Future<void> reportCrash(String errorName, String reason, String stacktrace,
      {Platform platform});

  /// Closes the session in the next possible moment.
  void endSession({Platform platform});

  /// Will attach a [user] to the session so you can identify the session later on.
  void identifyUser(String user, {Platform platform});

  /// Sets the GPS location of the session with [latitude] and [longitude].
  void setGPSLocation(double latitude, double longitude, {Platform platform});

  /// Returns the current value for crash reporting of the NATIVE agent.
  Future<bool> isCrashReportingOptedIn({Platform platform});

  /// Sets the [crashReporting] value for the NATIVE agent.
  void setCrashReportingOptedIn(bool crashReporting, {Platform platform});

  /// Returns the [DataCollectionLevel] for the NATIVE agent. Only valid when user opt in mode is used.
  /// If not used, it will return User mode per default.
  Future<DataCollectionLevel> getDataCollectionLevel({Platform platform});

  /// Set the data collection level for the NATIVE agent. Only valid when user opt in mode is used.
  void setDataCollectionLevel(DataCollectionLevel dataCollectionLevel,
      {Platform platform});

  /// Send events if necessary.
  void flushEvents({Platform platform});
}
