import 'dart:io';

import 'package:dynatrace_flutter_plugin/src/agent/model/log_level.dart';
import 'package:dynatrace_flutter_plugin/src/cli/config/model/cli_arguments.dart';
import 'package:yaml/yaml.dart';

import 'package:dynatrace_flutter_plugin/src/agent/util/logger.dart';
import 'package:dynatrace_flutter_plugin/src/cli/util/pathConstants.dart';
import 'package:dynatrace_flutter_plugin/src/cli/config/model/config.dart';
import 'package:dynatrace_flutter_plugin/src/cli/config/android_configurator.dart';
import 'package:dynatrace_flutter_plugin/src/cli/config/ios_configurator.dart';

/// Main setup configuration routine which accepts [arguments].
/// Currently they stay untouched as this will be fixed with another story.
Future<void> setupConfiguration(
    PathConstants paths, List<String> arguments) async {
  Logger().i("Dynatrace Flutter Plugin ${await getPluginVersion(paths)}");

  // Read CommandLine
  CommandLineArguments readArguments =
      CommandLineArguments.parseArgumentList(paths, arguments);

  Logger().i("Trying to read configuration file: " + readArguments.configPath);
  await checkIfConfigurationFileIsAvailable(readArguments);

  String contentYaml = await File(readArguments.configPath).readAsString();
  Configuration config = Configuration(contentYaml);
  Logger().logLevel = readArguments.isDebug ? LogLevel.Debug : LogLevel.Info;

  bool iosAvailable = await Directory(paths.getIosFolder()).exists();
  bool androidAvailable = await Directory(paths.getAndroidFolder()).exists();

  if (iosAvailable || androidAvailable) {
    // Try to read configuration
    try {
      // Try to instrument platform
      if (androidAvailable) {
        try {
          Logger().d("Starting Android Configuration with Dynatrace!");
          AndroidConfigurator configurator =
              AndroidConfigurator(paths, readArguments, null);
          await configurator.instrumentAndroidPlatform();
          await configurator
              .writeGradleConfig(config.getAndroidConfiguration());
        } catch (e) {
          Logger().i(e.message, logType: LogType.Error);
        } finally {
          Logger().d("Finished Android Configuration with Dynatrace!");
        }
      }

      if (iosAvailable) {
        try {
          Logger().d("Starting iOS Configuration with Dynatrace!");
          IosConfigurator configurator = IosConfigurator(readArguments, null);
          await configurator.modifyPListFile(config.getIosConfiguration());
        } catch (e) {
          Logger().i(e.message, logType: LogType.Error);
        } finally {
          Logger().d("Finished iOS Configuration with Dynatrace!");
        }
      }
    } catch (e) {
      Logger().i(e, logType: LogType.Error);
    }
  } else {
    Logger().d(
        "Both Android and iOS Folder are not available - Skip instrumentation.",
        logType: LogType.Warning);
  }

  Logger().i("Finished configuration of Flutter application ..");
}

/// Checks if the configuration file is available - if not it will throw an exception
Future<void> checkIfConfigurationFileIsAvailable(
    CommandLineArguments arguments) async {
  File config = File(arguments.configPath);

  if (await config.exists()) {
    Logger().d("Configuration found!");
  } else {
    Logger().i("dynatrace.config.yaml not found!", logType: LogType.Error);
    throw FileSystemException("dynatrace.config.yaml not found!");
  }
}

/// Returns the currently used plugin version read from the yaml file of the plugin
Future<String> getPluginVersion(PathConstants paths) async {
  String contentYaml = await File(paths.getPluginSpec()).readAsString();
  Map mapYaml = loadYaml(contentYaml);
  return mapYaml["version"];
}
